library(spider)
library(phangorn)

#Read in data
ausCoiM <- read.dna("coi.fasta", format = "fasta")
ausCoiBM <- read.dna("coib.fasta", format = "fasta")
aus28SM <- read.dna("28s.fasta", format = "fasta")
ausCadM <- read.dna("cad.fasta", format = "fasta")
ausArgk <- read.dna("argk.fasta", format = "fasta")

#Distance matrices
coiDist <- dist.dna(ausCoiM, model = "raw")
coiBDist <- dist.dna(ausCoiBM, model = "raw")
s28Dist <- dist.dna(aus28SM, model = "raw")
argkDist <- dist.dna(ausArgk, model = "raw")
cadDist <- dist.dna(ausCadM, model = "raw")
cadHDist <- dist.hamming(as.phyDat(ausCadM)) #Hamming distance to take into account polymorphisms

#Species vectors
vecTrans <- data.frame(tagName = c("Ausfurc", "Ausmata", "Ausatri", "Ausrotu", "Ausinfl"), pName = c("A. furcatus", "A. mataura", "A. atriarius", "A. rotundus", "A. inflatus"))

coiVec <- sapply(strsplit(dimnames(ausCoiM)[[1]], split = "\\|"), function(xx) xx[3])
coiVec <- vecTrans$pName[match(coiVec, vecTrans$tagName)]

coiBVec <- sapply(strsplit(dimnames(ausCoiBM)[[1]], split = "\\|"), function(xx) xx[3])
coiBVec <- vecTrans$pName[match(coiBVec, vecTrans$tagName)]

s28Vec <- sapply(strsplit(dimnames(aus28SM)[[1]], split = "\\|"), function(xx) xx[3])
s28Vec <- vecTrans$pName[match(s28Vec, vecTrans$tagName)]

argkVec <- sapply(strsplit(dimnames(ausArgk)[[1]], split = "\\|"), function(xx) xx[3])
argkVec <- vecTrans$pName[match(argkVec, vecTrans$tagName)]

cadVec <- sapply(strsplit(dimnames(ausCadM)[[1]], split = "\\|"), function(xx) xx[3])
cadVec <- vecTrans$pName[match(cadVec, vecTrans$tagName)]

bgplot <- function(distobj, sppVec, plotLabel = ""){
	inter <- nonConDist(distobj, sppVec) *100
	intra <- maxInDist(distobj, sppVec) *100
	min <- minInDist(distobj, sppVec) *100
	length(which(inter-intra <= 0))

	bnd <- cbind(data.frame(inter, intra, min, sppVec))
	#ord <- bnd
	ord <- bnd[order(bnd$sppVec),]
	plot(ord$inter, type="n", ylab="Uncorrected genetic distance (%)", xlab=NA, xaxt = "n", ylim = c(0, 12.5))
	segCol <- rep("grey60", length(ord$inter))
	segCol[ord$inter-ord$intra < 0] <- "red"
	segLine <- rep("15", length(ord$inter))
	segLine[ord$inter-ord$intra < 0] <- "solid"
	
	sppBGr <- cumsum(table(as.vector(sppVec))) + 0.5
	sppBGl <- c(0, sppBGr[-length(sppBGr)])
	rect(sppBGl, rep(-1, length(sppBGl)), sppBGr, rep(13, length(sppBGl)), col = c("white", "grey97"), border = NA)
	
	points(ord$min, pch = 8, col = "grey70", cex = 1.3)
	points(ord$intra, pch = 15, col = "grey50", cex = 1.3)
	points(ord$inter, pch = 19, col = "grey30", cex = 1.3)
	segments(x0=1:length(ord$inter), y0=ord$inter, y1=ord$intra, col=segCol, lty = segLine, lwd=2)
	legend("topleft", legend = plotLabel, bty = "n")
	axis(1, at = 1:length(ord$sppVec), labels = ord$sppVec, las = 2, cex.axis = 0.7)
	invisible(ord)
}

pdf(file = "AusBG_COI.pdf")
bgplot(coiDist, coiVec, plotLabel = "COI")
legend("topright", legend = c("Min. interspecific dist.", "Max. intraspecific dist.", "Min. intraspecific dist."), pch = c(19, 15, 8), col = c("grey30", "grey50", "grey70"), bty = "n")
dev.off()

pdf(file = "AusBG_COIB.pdf")
bgplot(coiBDist, coiBVec, plotLabel = "COIB")
dev.off()

pdf(file = "AusBG_28S.pdf")
bgplot(s28Dist, s28Vec, plotLabel = "28S")
dev.off()

pdf(file = "AusBG_ArgK.pdf")
bgplot(argkDist, argkVec, plotLabel = "ArgK")
dev.off()

pdf(file = "AusBG_CAD.pdf")
bgplot(cadDist, cadVec, plotLabel = "CAD")
# legend("topright", legend = c("Min. interspecific dist.", "Max. intraspecific dist.", "Min. intraspecific dist."), pch = c(19, 15, 8), col = c("grey30", "grey50", "grey70"), bty = "n")
dev.off()

pdf(file = "AusBG_CADH.pdf")
bgplot(cadHDist, cadVec, plotLabel = "CAD Hamming")
dev.off()

# Heatmaps

pdf(file = "AusHM_CAD.pdf")
par(mar = c(6,6,1,1) + 0.1)
heatmapSpp(cadDist, cadVec, showData = TRUE, triangle = "lower")
dev.off()

pdf(file = "AusHM_CADH.pdf")
par(mar = c(6,6,1,1) + 0.1)
heatmapSpp(cadHDist, cadVec, showData = TRUE, triangle = "lower")
dev.off()

pdf(file = "AusHM_COI.pdf")
par(mar = c(6,6,1,1) + 0.1)
heatmapSpp(coiDist, coiVec, showData = TRUE, triangle = "lower")
dev.off()

pdf(file = "AusHM_COIB.pdf")
par(mar = c(6,6,1,1) + 0.1)
heatmapSpp(coiBDist, coiBVec, showData = TRUE, triangle = "lower")
dev.off()

pdf(file = "AusHM_ArgK.pdf")
par(mar = c(6,6,1,1) + 0.1)
heatmapSpp(argkDist, argkVec, showData = TRUE, triangle = "lower")
dev.off()

pdf(file = "AusHM_28S.pdf")
par(mar = c(6,6,1,1) + 0.1)
heatmapSpp(s28Dist, s28Vec, showData = TRUE, triangle = "lower")
dev.off()


ndplot <- function(DNAbin, sppVec){
	DNAbin <- as.matrix(DNAbin)
	nd <- nucDiag(DNAbin, sppVec)
	nd <- nd[sort(names(nd))]
	lnd <- length(nd)
	end <- dim(DNAbin)[2]
	
	op <- par(mar = c(0, 0, 0, 0) + 0.5)
	plot(1, 1, type = "n", ylim = c(0, (lnd*3) + 2), xlim = c(0, end + 160), bty = "n", xaxt = "n", yaxt = "n", xlab = NA, ylab = NA)
	rect(rep(1, lnd), (1:lnd) * 3, rep(end, lnd), (1:lnd) * 3 + 1, col = "grey90", border = "grey90")
	text(rep(end + 1, lnd), (1:lnd) * 3 + 0.5, labels = paste(names(nd), " (", table(as.vector(sppVec)), ")", sep = ""), pos = 4)
	
	par(op)
	
	top <- 4
	bottom <- 3
	out <- list()
	length(out) <- lnd
	
	slashcoll <- function(vector) paste(unique(vector), collapse = "|")
	baseCol <- data.frame(base = c("a", "c", "g", "t", "t|c", "a|c", "g|a", "a|m", "a|r", "g|r", "t|y", "r", "y", "m"), col =c("#69FFB5", "#69B3FF", "#FFB569", "#FF69B3", rep("grey60", 10)), stringsAsFactors = FALSE)
	#baseCol <- data.frame(base = c("a", "c", "g", "t", "t|c", "a|c", "g|a", "a|m", "a|r", "g|r", "t|y"), col =c("#008000", "blue", "black", "red", rep("grey50", 7)), stringsAsFactors = FALSE)
	
	for(i in 1:lnd){
		if(length(nd[[i]]) > 0){
		ndcm <- as.character(DNAbin)[which(sppVec == names(nd)[i]) , nd[[i]]]
		if(class(ndcm) == "matrix") ndc <- unlist(apply(ndcm, 2, slashcoll)) else ndc <- ndcm
		#print(ndc)
		sites <- nd[[i]]
		ndcc <- baseCol$col[match(ndc, baseCol$base)]
		segments(sites, rep(bottom, length(sites)), sites, rep(top, length(sites)), col = ndcc, lwd = 2, lmitre = 1)
		text(sites, rep(top + 0.25, length(sites)) + rep_len(c(0.09, 0.00, -0.09), length(sites)), labels = sites, cex = 0.5)
		text(sites, rep(bottom - 0.25, length(sites)), labels = ndc, cex = 0.5)
		} else {
		text(end/2, bottom + 0.5, labels = "No diagnostic nucleotides")
		}
		top <- top + 3
		bottom <- bottom + 3
		
	}
	
	return(nd)
}

pdf(file = "AusND_COI.pdf")
coiDiag <- ndplot(ausCoiM, coiVec)
legend("topleft", legend = "COI", bty = "n")
dev.off()

pdf(file = "AusND_COIB.pdf")
coiBDiag <- ndplot(ausCoiBM, coiBVec)
legend("topleft", legend = "COI", bty = "n")
dev.off()

pdf(file = "AusND_28S.pdf")
s28Diag <- ndplot(aus28SM, s28Vec)
legend("topleft", legend = "28S", bty = "n")
dev.off()

pdf(file = "AusND_ArgK.pdf")
argkDiag <- ndplot(ausArgk, argkVec)
legend("topleft", legend = "ArgK", bty = "n")
dev.off()

pdf(file = "AusND_CAD.pdf")
cadDiag <- ndplot(ausCadM, cadVec)
legend("topleft", legend = "CAD", bty = "n")
dev.off()

summary(sapply(s28Diag, length))
summary(sapply(coiDiag, length))
summary(sapply(argkDiag, length))
summary(sapply(cadDiag, length))